/*
 * Copyright (c) 2008-2021 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.cometd.client.http;

import java.util.HashMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicReference;
import org.cometd.bayeux.Channel;
import org.cometd.bayeux.Message;
import org.cometd.bayeux.client.ClientSessionChannel;
import org.cometd.client.BayeuxClient;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

public class PublishDoesNotLeakClientIdTest extends ClientServerTest {
    @BeforeEach
    public void init() throws Exception {
        start(null);
    }

    @Test
    public void testPublishDoesNotLeakClientId() throws Exception {
        BayeuxClient client1 = newBayeuxClient();
        client1.handshake();
        try {
            Assertions.assertTrue(client1.waitFor(5000, BayeuxClient.State.CONNECTED));

            BayeuxClient client2 = newBayeuxClient();
            client2.handshake();
            try {
                Assertions.assertTrue(client2.waitFor(5000, BayeuxClient.State.CONNECTED));

                Assertions.assertNotEquals(client1.getId(), client2.getId());

                String channel = "/test";
                CountDownLatch subscribe = new CountDownLatch(1);
                client1.getChannel(Channel.META_SUBSCRIBE).addListener((ClientSessionChannel.MessageListener)(c, m) -> subscribe.countDown());
                CountDownLatch latch = new CountDownLatch(1);
                AtomicReference<Message> messageRef = new AtomicReference<>();
                client1.getChannel(channel).subscribe((c, m) -> {
                    messageRef.set(m);
                    latch.countDown();
                });
                Assertions.assertTrue(subscribe.await(5, TimeUnit.SECONDS));

                client2.getChannel(channel).publish(new HashMap<>());

                Assertions.assertTrue(latch.await(5, TimeUnit.SECONDS));
                Assertions.assertNull(messageRef.get().getClientId());
            } finally {
                disconnectBayeuxClient(client2);
            }
        } finally {
            disconnectBayeuxClient(client1);
        }
    }
}
